@@ -239,7 +239,9 @@ oonf_main(int argc, char **argv, const struct oonf_appdata *appdata) {
   }
 
   if (appdata->need_lock
-      && config_global.lockfile != NULL && *config_global.lockfile != 0) {
+      && config_global.lockfile != NULL
+      && *config_global.lockfile != 0
+      && strcmp(config_global.lockfile, "-") != 0) {
     /* create application lock */
     if (os_core_create_lockfile(config_global.lockfile)) {
       OONF_WARN(LOG_MAIN, "Could not acquire application lock '%s'",
