@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 /* must be first because of a problem with linux/rtnetlink.h */
 #include <sys/socket.h>
 
@@ -68,6 +72,7 @@
 #include <stdio.h>
 
 #ifndef SOL_NETLINK
+/*! socket netlink type */
 #define SOL_NETLINK 270
 #endif
 
@@ -78,9 +83,8 @@
 static int _init(void);
 static void _cleanup(void);
 
-static void _cb_handle_netlink_timeout(void *);
-static void _netlink_handler(int fd, void *data,
-    bool event_read, bool event_write);
+static void _cb_handle_netlink_timeout(struct oonf_timer_instance *);
+static void _netlink_handler(struct oonf_socket_entry *entry);
 static void _enqueue_netlink_buffer(struct os_system_netlink *nl);
 static void _handle_nl_err(struct os_system_netlink *, struct nlmsghdr *);
 static void _flush_netlink_buffer(struct os_system_netlink *nl);
@@ -190,15 +194,18 @@ _cleanup(void) {
  * @return true if IPv6 is supported, false otherwise
  */
 bool
-os_system_is_ipv6_supported(void) {
+os_system_linux_is_ipv6_supported(void) {
   return _ioctl_v6 != -1;
 }
 
 /**
+ * @param v1 first version number part
+ * @param v2 second version number part
+ * @param v3 third version number part
  * @return true if linux kernel is at least a specific version
  */
 bool
-os_linux_system_is_minimal_kernel(int v1, int v2, int v3) {
+os_system_linux_is_minimal_kernel(int v1, int v2, int v3) {
   struct utsname uts;
   char *next;
   int first = 0, second = 0, third = 0;
@@ -225,15 +232,15 @@ os_linux_system_is_minimal_kernel(int v1, int v2, int v3) {
   }
 
   second = strtol(next+1, &next, 10);
-  if (*next != '.') {
-    goto kernel_parse_error;
-  }
   if (second > v2) {
     return true;
   }
   if (second < v2) {
     return false;
   }
+  if (*next != '.') {
+    goto kernel_parse_error;
+  }
 
   third = strtol(next+1, NULL, 10);
   return third >= v3;
@@ -250,7 +257,7 @@ kernel_parse_error:
  * @return socket file descriptor, -1 if not surrported
  */
 int
-os_system_linux_get_ioctl_fd(int af_type) {
+os_system_linux_linux_get_ioctl_fd(int af_type) {
   switch (af_type) {
     case AF_INET:
       return _ioctl_v4;
@@ -268,17 +275,22 @@ os_system_linux_get_ioctl_fd(int af_type) {
  * @return -1 if an error happened, 0 otherwise
  */
 int
-os_system_netlink_add(struct os_system_netlink *nl, int protocol) {
+os_system_linux_netlink_add(struct os_system_netlink *nl, int protocol) {
   struct sockaddr_nl addr;
   int recvbuf;
+  int fd;
 
-  nl->socket.fd = socket(PF_NETLINK, SOCK_RAW, protocol);
-  if (nl->socket.fd < 0) {
+  fd = socket(PF_NETLINK, SOCK_RAW, protocol);
+  if (fd < 0) {
     OONF_WARN(nl->used_by->logging, "Cannot open netlink socket '%s': %s (%d)",
         nl->name, strerror(errno), errno);
     goto os_add_netlink_fail;
   }
 
+  if (os_fd_init(&nl->socket.fd, fd)) {
+    OONF_WARN(nl->used_by->logging, "Could not initialize socket representation");
+    goto os_add_netlink_fail;
+  }
   if (abuf_init(&nl->out)) {
     OONF_WARN(nl->used_by->logging, "Not enough memory for"
         " netlink '%s'output buffer", nl->name);
@@ -302,33 +314,31 @@ os_system_netlink_add(struct os_system_netlink *nl, int protocol) {
 
 #if defined(SO_RCVBUF)
   recvbuf = 65536*16;
-  if (setsockopt(nl->socket.fd, SOL_SOCKET, SO_RCVBUF,
+  if (setsockopt(nl->socket.fd.fd, SOL_SOCKET, SO_RCVBUF,
       &recvbuf, sizeof(recvbuf))) {
     OONF_WARN(nl->used_by->logging, "Cannot setup receive buffer size for"
         " netlink socket '%s': %s (%d)\n", nl->name, strerror(errno), errno);
   }
 #endif
 
-  if (bind(nl->socket.fd, (struct sockaddr *)&addr, sizeof(addr))<0) {
+  if (bind(nl->socket.fd.fd, (struct sockaddr *)&addr, sizeof(addr))<0) {
     OONF_WARN(nl->used_by->logging, "Could not bind netlink socket %s: %s (%d)",
         nl->name, strerror(errno), errno);
     goto os_add_netlink_fail;
   }
 
   nl->socket.process = _netlink_handler;
-  nl->socket.event_read = true;
-  nl->socket.data = nl;
   oonf_socket_add(&nl->socket);
+  oonf_socket_set_read(&nl->socket, true);
 
-  nl->timeout.cb_context = nl;
   nl->timeout.class = &_netlink_timer;
 
   list_init_head(&nl->buffered);
   return 0;
 
 os_add_netlink_fail:
-  if (nl->socket.fd != -1) {
-    close(nl->socket.fd);
+  if (fd != -1) {
+    close(fd);
   }
   free (nl->in);
   abuf_free(&nl->out);
@@ -340,14 +350,18 @@ os_add_netlink_fail:
  * @param nl pointer to handler
  */
 void
-os_system_netlink_remove(struct os_system_netlink *nl) {
+os_system_linux_netlink_remove(struct os_system_netlink *nl) {
   oonf_socket_remove(&nl->socket);
 
-  close(nl->socket.fd);
+  os_fd_close(&nl->socket.fd);
   free (nl->in);
   abuf_free(&nl->out);
 }
 
+/**
+ * add netlink message to buffer
+ * @param nl netlink message
+ */
 static void
 _enqueue_netlink_buffer(struct os_system_netlink *nl) {
   struct os_system_netlink_buffer *bufptr;
@@ -373,10 +387,10 @@ _enqueue_netlink_buffer(struct os_system_netlink *nl) {
  * @return sequence number used for message
  */
 int
-os_system_netlink_send(struct os_system_netlink *nl,
+os_system_linux_netlink_send(struct os_system_netlink *nl,
     struct nlmsghdr *nl_hdr) {
   _seq_used = (_seq_used + 1) & INT32_MAX;
-  OONF_INFO(nl->used_by->logging, "Prepare to send netlink '%s' message %u (%u bytes)",
+  OONF_DEBUG(nl->used_by->logging, "Prepare to send netlink '%s' message %u (%u bytes)",
       nl->name, _seq_used, nl_hdr->nlmsg_len);
 
   nl_hdr->nlmsg_seq = _seq_used;
@@ -405,12 +419,13 @@ os_system_netlink_send(struct os_system_netlink *nl,
  * @return -1 if an error happened, 0 otherwise
  */
 int
-os_system_netlink_add_mc(struct os_system_netlink *nl,
+os_system_linux_netlink_add_mc(struct os_system_netlink *nl,
     const uint32_t *groups, size_t groupcount) {
   size_t i;
 
   for (i=0; i<groupcount; i++) {
-    if (setsockopt(nl->socket.fd, SOL_NETLINK, NETLINK_ADD_MEMBERSHIP,
+    if (setsockopt(os_fd_get_fd(&nl->socket.fd),
+        SOL_NETLINK, NETLINK_ADD_MEMBERSHIP,
              &groups[i], sizeof(groups[i]))) {
       OONF_WARN(nl->used_by->logging,
           "Could not join netlink '%s' mc group: %x", nl->name, groups[i]);
@@ -428,12 +443,13 @@ os_system_netlink_add_mc(struct os_system_netlink *nl,
  * @return -1 if an error happened, 0 otherwise
  */
 int
-os_system_netlink_drop_mc(struct os_system_netlink *nl,
+os_system_linux_netlink_drop_mc(struct os_system_netlink *nl,
     const int *groups, size_t groupcount) {
   size_t i;
 
   for (i=0; i<groupcount; i++) {
-    if (setsockopt(nl->socket.fd, SOL_NETLINK, NETLINK_DROP_MEMBERSHIP,
+    if (setsockopt(os_fd_get_fd(&nl->socket.fd),
+        SOL_NETLINK, NETLINK_DROP_MEMBERSHIP,
              &groups[i], sizeof(groups[i]))) {
       OONF_WARN(nl->used_by->logging,
           "Could not drop netlink '%s' mc group: %x", nl->name, groups[i]);
@@ -453,7 +469,7 @@ os_system_netlink_drop_mc(struct os_system_netlink *nl,
  * @return -1 if netlink message got too large, 0 otherwise
  */
 int
-os_system_netlink_addreq(struct os_system_netlink *nl,
+os_system_linux_netlink_addreq(struct os_system_netlink *nl,
     struct nlmsghdr *nlmsg, int type, const void *data, int len) {
   struct nlattr *nl_attr;
   size_t aligned_msg_len, aligned_attr_len;
@@ -482,11 +498,13 @@ os_system_netlink_addreq(struct os_system_netlink *nl,
 
 /**
  * Handle timeout of netlink acks
- * @param ptr pointer to netlink handler
+ * @param ptr timer instance that fired
  */
 static void
-_cb_handle_netlink_timeout(void *ptr) {
-  struct os_system_netlink *nl = ptr;
+_cb_handle_netlink_timeout(struct oonf_timer_instance *ptr) {
+  struct os_system_netlink *nl;
+
+  nl = container_of(ptr, struct os_system_netlink, timeout);
 
   if (nl->cb_timeout) {
     nl->cb_timeout();
@@ -525,9 +543,14 @@ _flush_netlink_buffer(struct os_system_netlink *nl) {
   _netlink_send_iov[0].iov_base = (char *)(buffer) + sizeof(*buffer);
   _netlink_send_iov[0].iov_len = buffer->total;
 
-  if ((ret = sendmsg(nl->socket.fd, &_netlink_send_msg, MSG_DONTWAIT)) <= 0) {
+  if ((ret = sendmsg(os_fd_get_fd(&nl->socket.fd),
+        &_netlink_send_msg, MSG_DONTWAIT)) <= 0) {
     err = errno;
+#if EAGAIN == EWOULDBLOCK
+    if (err != EAGAIN) {
+#else
     if (err != EAGAIN && err != EWOULDBLOCK) {
+#endif
       OONF_WARN(nl->used_by->logging,
           "Cannot send data (%"PRINTF_SIZE_T_SPECIFIER" bytes)"
           " to netlink socket %s: %s (%d)",
@@ -541,7 +564,7 @@ _flush_netlink_buffer(struct os_system_netlink *nl) {
   else {
     nl->msg_in_transit += buffer->messages;
 
-    OONF_INFO(nl->used_by->logging,
+    OONF_DEBUG(nl->used_by->logging,
         "netlink %s: Sent %u bytes (%u messages in transit)",
         nl->name, buffer->total, nl->msg_in_transit);
 
@@ -585,7 +608,7 @@ _netlink_job_finished(struct os_system_netlink *nl) {
  * @param event_write
  */
 static void
-_netlink_handler(int fd, void *data, bool event_read, bool event_write) {
+_netlink_handler(struct oonf_socket_entry *entry) {
   struct os_system_netlink *nl;
   struct nlmsghdr *nh;
   ssize_t ret;
@@ -594,12 +617,12 @@ _netlink_handler(int fd, void *data, bool event_read, bool event_write) {
   uint32_t current_seq = 0;
   bool trigger_is_done;
 
-  nl = data;
-  if (event_write) {
+  nl = container_of(entry, typeof(*nl), socket);
+  if (oonf_socket_is_write(entry)) {
     _flush_netlink_buffer(nl);
   }
 
-  if (!event_read) {
+  if (!oonf_socket_is_read(entry)) {
     return;
   }
 
@@ -614,8 +637,12 @@ netlink_rcv_retry:
   OONF_DEBUG(nl->used_by->logging, "Read netlink '%s' message with"
       " %"PRINTF_SIZE_T_SPECIFIER" bytes buffer",
       nl->name, nl->in_len);
-  if ((ret = recvmsg(fd, &_netlink_rcv_msg, MSG_DONTWAIT | flags)) < 0) {
+  if ((ret = recvmsg(entry->fd.fd, &_netlink_rcv_msg, MSG_DONTWAIT | flags)) < 0) {
+#if EAGAIN == EWOULDBLOCK
+    if (errno != EAGAIN) {
+#else
     if (errno != EAGAIN && errno != EWOULDBLOCK) {
+#endif
       OONF_WARN(nl->used_by->logging,"netlink '%s' recvmsg error: %s (%d)\n",
           nl->name, strerror(errno), errno);
     }
@@ -651,7 +678,7 @@ netlink_rcv_retry:
     goto netlink_rcv_retry;
   }
 
-  OONF_INFO(nl->used_by->logging, "Got netlink '%s' message of %"
+  OONF_DEBUG(nl->used_by->logging, "Got netlink '%s' message of %"
       PRINTF_SSIZE_T_SPECIFIER" bytes", nl->name, ret);
   OONF_DEBUG_HEX(nl->used_by->logging, nl->in, ret,
       "Content of netlink '%s' message:", nl->name);
@@ -661,7 +688,7 @@ netlink_rcv_retry:
   /* loop through netlink headers */
   len = (size_t) ret;
   for (nh = nl->in; NLMSG_OK (nh, len); nh = NLMSG_NEXT (nh, len)) {
-    OONF_INFO(nl->used_by->logging,
+    OONF_DEBUG(nl->used_by->logging,
         "Netlink '%s' message received: type %d seq %u\n",
         nl->name, nh->nlmsg_type, nh->nlmsg_seq);
 
@@ -724,7 +751,7 @@ _handle_nl_err(struct os_system_netlink *nl, struct nlmsghdr *nh) {
 
   err = (struct nlmsgerr *) NLMSG_DATA(nh);
 
-  OONF_INFO(nl->used_by->logging,
+  OONF_DEBUG(nl->used_by->logging,
       "Received netlink '%s' seq %u feedback (%u bytes): %s (%d)",
       nl->name, nh->nlmsg_seq, nh->nlmsg_len, strerror(-err->error), -err->error);
 
